@@ -43,15 +43,20 @@ to the ``request`` object, so that it can be used in the application code::
     async def home_handler(request):
         ctx = request['datadog_context']
         # do something with the tracing Context
+
+:ref:`All HTTP tags <http-tagging>` are supported for this integration.
+
 """
 from ...utils.importlib import require_modules
 
+
 required_modules = ["aiohttp"]
 
 with require_modules(required_modules) as missing_modules:
     if not missing_modules:
-        from .patch import patch, unpatch
         from .middlewares import trace_app
+        from .patch import patch
+        from .patch import unpatch
 
         __all__ = [
             "patch",
